@@ -28,10 +28,11 @@ from __future__ import annotations
 
 __title__ = "asyncnt"
 __author__ = "axemalt"
-__version__ = "1.4.2"
+__version__ = "1.6.0"
 
 
 from typing import Optional, Union, Type, List, Dict
+from collections import OrderedDict
 from types import TracebackType
 import asyncio
 import aiohttp
@@ -45,6 +46,7 @@ class AsyncNTException(Exception):
 
     pass
 
+
 class InvalidArgument(AsyncNTException):
     """Exception that is raised when an argument provided is invalid."""
 
@@ -69,6 +71,14 @@ class InvalidTeamTag(AsyncNTException):
         super().__init__(message)
 
 
+class InvalidID(AsyncNTException):
+    """Exception that is raised when the ID provided is invalid."""
+
+    def __init__(self) -> None:
+        message = "The ID provided is invalid."
+        super().__init__(message)
+
+
 class HTTPException(AsyncNTException):
     """Exception that is raised when an HTTP request operation fails."""
 
@@ -84,29 +94,137 @@ class HTTPException(AsyncNTException):
         super().__init__(fmt.format(self.response, self.code))
 
 
+class _Cache:
+    __slots__ = [
+        "cache",
+        "maxsize",
+        "cache_for",
+        "tasks",
+    ]
+
+    def __init__(self, cache_for: Union[float, int], maxsize: int = 128) -> None:
+        error_message = "a float or int greater or equal to 0"
+        if not isinstance(cache_for, (int, float)):
+            raise InvalidArgument("cache_for", error_message)
+        if cache_for < 0:
+            raise InvalidArgument("cache_for", error_message)
+
+        error_message = "an int greater or equal to 0"
+        if not isinstance(maxsize, int):
+            raise InvalidArgument("cache_maxsize", error_message)
+        if maxsize < 0:
+            raise InvalidArgument("cache_maxsize", error_message)
+
+        self.cache: OrderedDict[str, aiohttp.ClientResponse] = OrderedDict()
+        self.maxsize: int = cache_for
+        self.cache_for: float = maxsize
+        self.tasks: List[asyncio.Task] = []
+
+    def __del__(self) -> None:
+        for task in self.tasks:
+            task.cancel()
+
+    def get(self, url: str) -> aiohttp.ClientResponse:
+        return self.cache.get(url)
+
+    def clear(self) -> None:
+        self.cache = OrderedDict()
+
+    def add(self, url: str, response: aiohttp.ClientResponse) -> None:
+        if self.cache_for != 0 and self.maxsize != 0:
+            self.cache[url] = response
+            task = asyncio.create_task(self.auto_remove(url))
+            self.tasks.append(task)
+
+            if len(self.cache) > self.maxsize:
+                self.cache.popitem(last=False)
+
+    def remove(self, url: str) -> None:
+        try:
+            self.cache.pop(url)
+        except KeyError:
+            pass
+
+    async def auto_remove(self, url: str):
+        await asyncio.sleep(self.cache_for)
+        self.remove(url)
+
+
+class _RateLimit:
+    __slots__ = [
+        "rate",
+        "per",
+        "window",
+        "tokens",
+        "lock",
+    ]
+
+    def __init__(self, rate: int = 10, per: Union[float, int] = 1) -> None:
+        error_message = "an int greater than 0"
+        if not isinstance(rate, int):
+            raise InvalidArgument("rate", error_message)
+        if rate <= 0:
+            raise InvalidArgument("rate", error_message)
+
+        error_message = "a float or int greater or equal to 0"
+        if not isinstance(per, (int, float)):
+            raise InvalidArgument("limit_for", error_message)
+        if per < 0:
+            raise InvalidArgument("limit_for", error_message)
+
+        self.lock = asyncio.Lock()
+        self.rate: int = rate
+        self.per: float = per
+        self.window: float = 0.0
+        self.tokens: int = self.rate
+
+    def update(self) -> Optional[float]:
+        current = time.time()
+
+        if current > self.window + self.per:
+            self.tokens = self.rate
+            self.window = current
+
+        if self.tokens == 0:
+            return self.per - (current - self.window)
+
+        self.tokens -= 1
+
+    async def wait(self) -> None:
+        if self.per != 0:
+            async with self.lock:
+                wait_for = self.update()
+                if wait_for:
+                    await asyncio.sleep(wait_for)
+                    self.update()
+
+
 class Car:
     """Represents a Nitro Type car."""
 
-    __slots__ = ["id", "hue_angle", "url"]
+    __slots__ = ["id", "name", "description", "rarity", "url", "price", "enter_sound"]
 
     def __init__(self, data):
-        #: The car's id.
-        self.id: int = data[0]
-        #: The car's hue angle.
-        self.hue_angle: int = data[2]
-        #: The car's image url.
-        self.url: str = ""
+        options = data["options"]
 
-        if self.hue_angle == 0:
-            self.url = f"https://www.nitrotype.com/cars/{self.id}_large_1.png"
-        else:
-            self.url = f"https://www.nitrotype.com/cars/painted/{self.id}_large_1_{self.hue_angle}.png"
+        #: The car's id.
+        self.id: int = data["carID"]
+        #: The car's name.
+        self.name: str = data["name"]
+        #: The car's description.
+        self.description: str = data["longDescription"]
+        #: The car's enter sound.
+        self.enter_sound: str = data["enterSound"]
+        #: The car's rarity.
+        self.rarity: str = options["rarity"]
+        #: The car's image url.
+        self.url: str = options["largeSrc"]
 
 
 class Loot:
     """Represents a Nitro Type loot."""
 
-    __slots__ = ["id", "type", "name", "rarity"]
+    __slots__ = ["id", "type", "name", "description", "rarity", "price"]
 
     def __init__(self, data):
         #: The loot's ID.
@@ -115,8 +233,12 @@ class Loot:
         self.type: str = data["type"]
         #: The loot's name.
         self.name: str = data["name"]
+        #: The loot's price.
+        self.price: Optional[int] = data.get("price")
         #: The loot's rarity.
-        self.rarity: str = data["options"]["rarity"]
+        self.rarity = None
+        if data.get("options") is not None:
+            self.rarity: str = data["options"]["rarity"]
 
 
 class Racer:
@@ -126,6 +248,8 @@ class Racer:
         "id",
         "username",
         "display_name",
+        "url",
+        "tag_and_name",
         "membership",
         "level",
         "experience",
@@ -140,17 +264,20 @@ class Racer:
         "friend_reqs_allowed",
         "looking_for_team",
         "created",
-        "car",
-        "cars",
         "cars_owned",
         "cars_sold",
         "cars_total",
-        "loot",
         "_scraper",
+        "_car_id",
+        "_car_ids",
+        "_loot_ids",
     ]
 
     def __init__(self, data: Dict, *, scraper: Session) -> None:
         self._scraper = scraper
+        self._car_id = data["carID"]
+        self._car_ids: List[Car] = []
+        self._loot_ids: List[Optional[Loot]] = []
 
         #: The racer's user ID.
         self.id: int = data["userID"]
@@ -158,6 +285,14 @@ class Racer:
         self.username: str = data["username"]
         #: The racer's display name.
         self.display_name: str = data["displayName"] or self.username
+        #: The racer's profile url.
+        self.url: str = f"https://nitrotype.com/racer/{self.username}"
+        #: The racer's team tag. ``None`` if the racer has no team.
+        self.team_tag: Optional[str] = data["tag"]
+        #: The racer's tag and name. If the racer has no team, this is the same as the display name.
+        self.tag_and_name = self.display_name
+        if self.team_tag:
+            self.tag_and_name = f"[{self.team_tag}] {self.display_name}"
 
         #: The racer's membership (gold, basic).
         self.membership: str = data["membership"]
@@ -178,9 +313,6 @@ class Racer:
         #: The racer's amount of races.
         self.races: int = data["racesPlayed"]
 
-        #: The racer's team tag. ``None`` if the racer has no team.
-        self.team_tag: Optional[str] = data["tag"]
-
         #: The racer's average speed.
         self.average_speed: int = data["avgSpeed"]
         #: The racer's highest speed.
@@ -194,33 +326,63 @@ class Racer:
         #: The racer's creation time.
         self.created: int = data["createdStamp"]
 
-        #: The racer's current car.
-        self.car: Car = Car([data["carID"], "owned", data["carHueAngle"]])
-
         #: The racer's amount of owned cars.
         self.cars_owned: int = 0
         #: The racer's amount of sold cars.
         self.cars_sold: int = 0
         #: The racer's amount of owned and sold cars.
         self.cars_total: int = 0
-        #: The racer's cars.
-        self.cars: List[Car] = []
         for car in data["cars"]:
             if car[1] == "owned":
-                self.cars.append(Car(car))
+                self._car_ids.append(car[0])
                 self.cars_owned += 1
             elif car[1] == "sold":
                 self.cars_sold += 1
             self.cars_total += 1
 
-        #: The racer's loot.
-        self.loot: List[Optional[Loot]] = []
         for loot in data["loot"]:
-            self.loot.append(Loot(loot))
+            self._loot_ids.append(loot["lootID"])
+
+    async def get_car(self) -> Car:
+        """
+        Returns the racer's equipped car.
+
+        :raise asyncnt.HTTPException: Getting the car failed.
+        :return: The racer's equipped car.
+        :rtype: asyncnt.Car
+        """
+
+        return await self._scraper.get_car(self._car_id)
+
+    async def get_cars(self) -> List[Car]:
+        """
+        Returns the racer's cars.
+
+        :raise asyncnt.HTTPException: Getting the cars failed.
+        :return: The racer's cars.
+        :rtype: List[asyncnt.Car]
+        """
+
+        return await asyncio.gather(
+            *[self._scraper.get_car(id) for id in self._car_ids]
+        )
+
+    async def get_loot(self) -> List[Loot]:
+        """
+        Returns the racer's loot.
+
+        :raise asyncnt.HTTPException: Getting the loot failed.
+        :return: The racer's loot.
+        :rtype: List[asyncnt.Loot]
+        """
+
+        return await asyncio.gather(
+            *[self._scraper.get_loot(id) for id in self._loot_ids]
+        )
 
     async def get_team(self) -> Optional[Team]:
         """
-        Return the team the racer is on.
+        Returns the team the racer is on.
 
         :raise asyncnt.HTTPException: Getting the team failed.
         :return: The racer's team. ``None`` if the racer has no team.
@@ -239,6 +401,8 @@ class Team:
         "id",
         "tag",
         "name",
+        "url",
+        "tag_and_name",
         "open",
         "created",
         "profile_views",
@@ -277,6 +441,10 @@ class Team:
         self.tag: str = info["tag"]
         #: The team's name.
         self.name: str = info["name"]
+        #: The team's profile url.
+        self.url: str = f"https://nitrotype.com/racer/{self.tag}"
+        #: The team's tag and name.
+        self.tag_and_name = f"[{self.tag}] {self.name}"
 
         #: Whether the team allows new members.
         self.open: bool = info["enrollment"] == "open"
@@ -401,35 +569,44 @@ class Team:
 
 
 class Session:
-    """First-class interface for making HTTP requests to Nitro Type."""
+    """
+    First-class interface for making HTTP requests to Nitro Type.
+
+    :param rate: The number of tokens available per :attr:`asyncnt.Session.limit_for` seconds. Defaults to 10.
+    :type rate: int
+    :param limit_for: The length of the rate limit in seconds. If 0, there is no rate limit. Defaults to 1.
+    :type limit_for: Union[float, int]
+    :param cache_for: The amount of time in seconds to cache results for. If 0, results will not be cached. Defaults to 300.
+    :type cache_for: Union[float, int]
+    :param cache_maxsize: The maximum size of the cache. If 0, results will not be cached. Defaults to 128.
+    :type cache_maxsize: int
+    :raises asyncnt.InvalidArgument: One of the arguments provided is invalid.
+    """
 
     __slots__ = [
-        "_limit_for",
-        "_rate_limit",
-        "_cache_for",
-        "_lock",
         "_loop",
         "_cache",
-        "_window",
-        "_tokens",
         "_session",
+        "_ratelimit",
     ]
 
-    def __init__(self) -> None:
+    def __init__(
+        self,
+        *,
+        rate: int = 10,
+        limit_for: Union[float, int] = 1,
+        cache_for: Union[float, int] = 300,
+        cache_maxsize: int = 128,
+    ) -> None:
+
         headers = {
             "User-Agent": "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1941.0 Safari/537.36",
         }
 
-        self._limit_for: float = 1
-        self._rate_limit: int = 10
-        self._cache_for: float = 300
-
-        self._lock: asyncio.Lock = asyncio.Lock()
-        self._loop = asyncio.get_event_loop()
-        self._cache: Dict[str, aiohttp.ClientResponse] = {}
-        self._window: float = 0.0
-        self._tokens: int = self._rate_limit
+        self._loop: asyncio.AbstractEventLoop = asyncio.get_event_loop()
+        self._cache: _Cache = _Cache(cache_for, cache_maxsize)
         self._session: aiohttp.ClientSession = aiohttp.ClientSession(headers=headers)
+        self._ratelimit: _RateLimit = _RateLimit(rate, limit_for)
 
     def __del__(self) -> None:
         if not self._session.closed:
@@ -447,96 +624,6 @@ class Session:
 
         await self._session.close()
 
-    @property
-    def rate_limit(self) -> int:
-        """The number of tokens available per :attr:`asyncnt.Session.limit_for` seconds. Defaults to 10."""
-
-        return self._rate_limit
-
-    def set_rate_limit(self, value: int) -> None:
-        """
-        Set the number of tokens available per :attr:`asyncnt.Session.limit_for` seconds.
-        
-        :param value: The amount of tokens.
-        :type value: int
-        :raise asyncnt.InvalidArgument: The value provided is invalid.
-        """
-
-        error_message = "an int greater than 0"
-
-        if not isinstance(value, int):
-            raise InvalidArgument("rate_limit", error_message)
-        if value <= 0:
-            raise InvalidArgument("rate_limit", error_message)
-
-        self._rate_limit = value
-
-    @property
-    def limit_for(self) -> Union[float, int]:
-        """The length of the rate limit in seconds. Defaults to 1."""
-
-        return self._limit_for
-
-    def set_limit_for(self, value: Union[float, int]) -> None:
-        """
-        Set the length of the rate limit in seconds. If the value is 0, there is no rate limit.
-        
-        :param value: The amount of seconds the rate limit lasts.
-        :type value: Union[float, int]
-        :raise asyncnt.InvalidArgument: The value provided is invalid.
-        """
-
-        error_message = "a float or int greater or equal to 0"
-
-        if not isinstance(value, (int, float)):
-            raise InvalidArgument("limit_for", error_message)
-        if value < 0:
-            raise InvalidArgument("limit_for", error_message)
-
-        self._limit_for = value
-
-    @property
-    def cache_for(self) -> Union[float, int]:
-        """The amount of time in seconds to cache results for. Defaults to 300."""
-
-        return self._cache_for
-
-    def set_cache_for(self, value: Union[float, int]) -> None:
-        """
-        Set the amount of time in seconds to cache results. If the value is 0, results will not be cached.
-        
-        :param value: The amount of seconds to cache results for.
-        :type value: Union[float, int]
-        :raise asyncnt.InvalidArgument: The value provided is invalid.
-        """
-        
-        error_message = "a float or int greater or equal to 0"
-
-        if not isinstance(value, (int, float)):
-            raise InvalidArgument("cache_for", error_message)
-        if value < 0:
-            raise InvalidArgument("cache_for", error_message)
-
-        if value == 0:
-            self._cache = {}
-        self._cache_for = value
-
-    def _update_rate_limit(self) -> Optional[float]:
-        current = time.time()
-
-        if current > self._window + self._limit_for:
-            self._tokens = self._rate_limit
-            self._window = current
-
-        if self._tokens == 0:
-            return self._limit_for - (current - self._window)
-
-        self._tokens -= 1
-
-    async def _remove_from_cache(self, url: str) -> None:
-        await asyncio.sleep(self._cache_for)
-        self._cache.pop(url)
-
     async def get(self, url: str) -> Optional[aiohttp.ClientResponse]:
         """
         Gets data from Nitro Type.
@@ -551,24 +638,30 @@ class Session:
         if result := self._cache.get(url):
             return result
 
-        if self._limit_for != 0:
-            async with self._lock:
-                wait_for = self._update_rate_limit()
-                if wait_for:
-                    await asyncio.sleep(wait_for)
-                    self._update_rate_limit()
+        await self._ratelimit.wait()
 
         response: aiohttp.ClientResponse = await self._session.get(url)
 
         if response.status == 200:
-            if self._cache_for != 0:
-                self._cache[url] = response
-                self._loop.create_task(self._remove_from_cache(url))
-                
+            self._cache.add(url, response)
             return response
         else:
             raise HTTPException(response)
 
+    async def get_boostrap(self) -> aiohttp.ClientResponse:
+        """
+        A shortcut to getting data from the bootstrap.
+
+        :raise asyncnt.HTTPException: Getting the data failed.
+        :return: The boostrap data.
+        :rtype: aiohttp.ClientResponse
+        """
+
+        raw_data: aiohttp.ClientResponse = await self.get(
+            "https://www.nitrotype.com/index/d8dad03537419610ef21782a075dde2d94c465c61266-1266/bootstrap.js"
+        )
+        return raw_data
+
     async def get_racer(self, username: str) -> Optional[Racer]:
         """
         Get a racer with a username.
@@ -615,3 +708,52 @@ class Session:
             raise InvalidTeamTag
 
         return Team(data["data"], scraper=self)
+
+    async def get_car(self, id: int) -> Optional[Car]:
+        """
+        Get a car with an ID.
+
+        :param id: The car's ID.
+        :type id: int
+        :raise asyncnt.InvalidID: The ID given is invalid.
+        :raise asyncnt.HTTPException: Getting the car failed.
+        :return: The car with the given ID.
+        :rtype: Optional[asyncnt.Car]
+        """
+
+        raw_data = await self.get_boostrap()
+        text = await raw_data.text()
+
+        regex_result: str = re.search(
+            r"\"CARS\",(\[.*\])\],\[\"PRODUCTS\"", text
+        ).group(1)
+        car_data = json.loads('{"cars": ' + regex_result + "}")
+
+        for car in car_data["cars"]:
+            if car["carID"] == id:
+                return Car(car)
+
+        raise InvalidID
+
+    async def get_loot(self, id: int) -> Optional[Loot]:
+        """
+        Get loot with an ID.
+        :param id: The loot's ID.
+        :type id: int
+        :raise asyncnt.InvalidLootID: The ID given is invalid.
+        :raise asyncnt.HTTPException: Getting the loot failed.
+        :return: The loot with the given ID.
+        :rtype: Optional[asyncnt.Loot]
+        """
+
+        raw_data = await self.get_boostrap()
+        text = await raw_data.text()
+
+        regex_result: str = re.search(r"\"LOOT\",(\[.*\])\],\[\"SHOP\"", text).group(1)
+        loot_data = json.loads('{"loot": ' + regex_result + "}")
+
+        for loot in loot_data["loot"]:
+            if loot["lootID"] == id:
+                return Loot(loot)
+
+        raise InvalidID
